#include <avr/interrupt.h>
#include <util/delay.h>
#include "main.h"
#include "pwm.h"
#include "dimm_array.h"

#define DIMM_STEPS      (1<<DIMM_BITS)

void set_pattern( uint16_t state )
{
  for( uint8_t i = 0; i < 8; i++ ){
    uint8_t val;

    switch( state ){
      case 0 ... DIMM_STEPS-1:                                          //  0 ... 15
                                        val = state;
                                        break;
      case DIMM_STEPS ... DIMM_STEPS*2-1:                               // 16 ... 31
                                        val = DIMM_STEPS-1;
                                        break;
      case DIMM_STEPS*2 ... DIMM_STEPS*3-1:                             // 32 ... 47
                                        val = DIMM_STEPS*3-1-state;
                                        break;
      default:
                                        val = 0;
    }
    pwm_val[i] = dimm( val );
    state -= DIMM_STEPS;
  }
  pwm_set();
  _delay_ms( DELAY_MS );
}

int main()
{
  uint16_t steps = DIMM_STEPS;
  
  pwm_init();
  sei();
  for(;;){
    do
      set_pattern( steps );
    while( ++steps < DIMM_STEPS * 9 - 1 );
    do
      set_pattern( steps );
    while( --steps > DIMM_STEPS);
  }
}
