#include <avr/io.h>
#include "main.h"
#include "pwm.h"

#if PWM_BITS < 9 || PWM_BITS > 12
#error Only 9 .. 12 Bit PWM implemented !!!
#endif

.macro  delay reg, val                  ;delay 0 .. 770 cycle
.if     \val > 2
        ldi     \reg, \val / 3
1:
        dec     \reg
        brne    1b
.endif
.if     (\val % 3) == 1
        nop
.endif
.if     (\val % 3) == 2
        rjmp    .+0
.endif
.endm

.comm pwm_data, PWM_BITS

;-------------------------------------------------------------------------------
.equ    PWM_OCR1BL, (0xFE - 17 + 11)            ;time difference to data out
.global TIMER1_COMPA_vect
TIMER1_COMPA_vect:                              ;Bit 7..0
        push    r16
        lds     r16, TCNT1L                     ;= 9 .. 12
        push    r17
        in      r17, _SFR_IO_ADDR(SREG)
        subi    r16, 9
        sbrs    r16, 0                          ;compensate latency
        rjmp    .+0                             ;add 1 cycle
        sbrs    r16, 1
        lpm     r16, z                          ;add 2 cycle
        push    r18
        push    r19
        lds     r16, pwm_data
        lds     r18, pwm_data+1
        lds     r19, pwm_data+2
        out     _SFR_IO_ADDR(PWM_PORT), r16     ;11 cycle after latency compensation
        out     _SFR_IO_ADDR(PWM_PORT), r18     ;1 cycle
        nop
        out     _SFR_IO_ADDR(PWM_PORT), r19     ;2 cycle
        lds     r16, pwm_data+3
        nop
        out     _SFR_IO_ADDR(PWM_PORT), r16     ;4 cycle
        pop     r19
        lpm     r16, z                          ;delay 3
        lds     r16, pwm_data+4
        out     _SFR_IO_ADDR(PWM_PORT), r16     ;8 cycle
        delay   r16, 13
        lds     r16, pwm_data+5
        out     _SFR_IO_ADDR(PWM_PORT), r16     ;16 cycle
        delay   r16, 27
        pop     r18
        lds     r16, pwm_data+6
        out     _SFR_IO_ADDR(PWM_PORT), r16     ;32 cycle
        delay   r16, 52
        out     _SFR_IO_ADDR(SREG), r17         ;restore SREG
        pop     r17
        ldi     r16, 0
        sts     OCR1BH, r16                     ;write to temp register
        ldi     r16, PWM_OCR1BL
        sts     OCR1BL, r16                     ;write 16 bit to OCR1A
        lds     r16, pwm_data+7
        out     _SFR_IO_ADDR(PWM_PORT), r16     ;64 cycle
        pop     r16
        reti
;-------------------------------------------------------------------------------
.global TIMER1_COMPB_vect
TIMER1_COMPB_vect:                              ;handle upper bits
        push    r16
        lds     r16, TCNT1L                     ;= 9 .. 12
        push    r17
        in      r17, _SFR_IO_ADDR(SREG)
        subi    r16, (PWM_OCR1BL + 1 + 9) % 4
        sbrs    r16, 0                          ;compensate latency
        rjmp    .+0                             ;add 1 cycle
        sbrs    r16, 1
        lpm     r16, z                          ;add 2 cycle
        lds     r16, OCR1BH
        sec
        rol     r16                             ;cycle until next interrupt
        out     _SFR_IO_ADDR(SREG), r17
        lds     r17, pwm_data+8
        sbrc    r16, 1
        lds     r17, pwm_data+9
        sbrc    r16, 2
        lds     r17, pwm_data+10
        sbrc    r16, 3
        lds     r17, pwm_data+11
        out     _SFR_IO_ADDR(PWM_PORT), r17     ;17 cycle after latency compensation
        ldi     r17, PWM_OCR1BL
        sts     OCR1BH, r16                     ;write to temp register
        sbrs    r16, PWM_BITS - 9               ;all bits done: skip new value
        sts     OCR1BL, r17                     ;write 16 bit to OCR1A
        pop     r17
        pop     r16
        reti
;-------------------------------------------------------------------------------
.end
